/************************************************************/
/*                                                          */
/*                     CLEANUP HEADER FILE                  */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODULE     : CbfClean.h                             */
/*      AUTHOR     :                                        */
/*      LABORATORY : L.A.P.P.                               */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODIFICATION:                                       */
/*                                                          */
/************************************************************/
/*                                                          */
/*                                                          */
/************************************************************/
/* Copyright (c) 2000 LAPP Annecy, UMR-IN2P3-CNRS (France). */
/*                                                          */
/* Redistribution and use in source and binary forms, with  */
/* or without modification, are permitted provided that the */
/* following conditions are met:                            */
/* 1. Redistributions of source code must retain the above  */
/*    copyright notice, this list of conditions and the     */
/*    following disclaimer.                                 */
/* 2. Redistributions in binary form must reproduce the     */
/*    above copyright notice, this list of conditions and   */
/*    the following disclaimer in the documentation and/or  */
/*    other materials provided with the distribution.       */
/* 3. All advertising materials mentioning features or use  */
/*    of this software must display the following           */
/*    acknowledgement:                                      */
/*      This product includes software developed by the     */
/*      Computer Application Development Group at LAPP      */
/*      Annecy  (Laboratoire d'Annecy le vieux de Physique  */
/*      des Particules - UMR-IN2P3-CNRS).                   */
/* 4. Neither the name of the Institute nor of the          */
/*    Laboratory may be used to endorse or promote products */
/*    derived from this software without specific prior     */
/*    written permission.                                   */
/*                                                          */
/* THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   */
/* ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY */
/* OF SUCH DAMAGE.                                          */
/************************************************************/


#include <CbfHash.h>

#ifndef CbfClean_h
#define CbfClean_h

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CBFCLEAN_MAX 128
typedef void *(*CbfCleanFnt_t)(void *);
typedef void (*CbfCleanExitFnt_t)(void);
typedef int (*CbfCleanPrinter_t) (const char* format, va_list args);

typedef struct CbfCleanChain {
  char name[CBFCLEAN_MAX];
  CbfCleanFnt_t function;
  void *arg;
  int  loop;
  struct CbfClean *cleanRef;
  struct CbfCleanChain *next;
} CbfCleanChain_t;

typedef struct CbfClean {
  CbfCleanChain_t *chain;
  char name[CBFCLEAN_MAX];
  pthread_mutex_t mutex;
} CbfClean_t;


extern CbfCleanChain_t *CbfCleanAdd( CbfClean_t *clean, char *fName,
				     CbfCleanFnt_t function, void *arg );
#define CbfCleanAddRoot( fName, function, arg ) \
        CbfCleanAdd((CbfClean_t*)NULL, fName, function, arg)
extern int CbfCleanDone( char *fName, void *arg );
extern CbfCleanChain_t *CbfCleanFind( char *fName, void *arg );
extern CbfClean_t *CbfCleanFree( CbfClean_t *clean );
extern CbfClean_t *CbfCleanFreeTag( char *fName, void *arg );
extern CbfCleanPrinter_t CbfCleanInstallPrinter( CbfCleanPrinter_t printer );
extern CbfClean_t *CbfCleanNew( char *name );
extern void CbfCleanPrint( CbfClean_t *clean, char *format, ... );
extern void CbfCleanSetDebug( int debug, char *name );
extern CbfCleanExitFnt_t CbfCleanSetExitFnt( CbfCleanExitFnt_t exitFnt );
extern int CbfCleanFinished(void);
extern void CbfTerminate(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* CbfSearch_h */
