/************************************************************/
/*                                                          */
/*                  HASH TABLE HEADER FILE                  */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODULE     : CbfHash.h                              */
/*      AUTHOR     :                                        */
/*      LABORATORY : L.A.P.P.                               */
/*                                                          */
/************************************************************/
/*                                                          */
/*      MODIFICATION:                                       */
/*                                                          */
/************************************************************/
/*                                                          */
/*                                                          */
/************************************************************/
/* Copyright (c) 2001 LAPP Annecy, UMR-IN2P3-CNRS (France). */
/*                                                          */
/* Redistribution and use in source and binary forms, with  */
/* or without modification, are permitted provided that the */
/* following conditions are met:                            */
/* 1. Redistributions of source code must retain the above  */
/*    copyright notice, this list of conditions and the     */
/*    following disclaimer.                                 */
/* 2. Redistributions in binary form must reproduce the     */
/*    above copyright notice, this list of conditions and   */
/*    the following disclaimer in the documentation and/or  */
/*    other materials provided with the distribution.       */
/* 3. All advertising materials mentioning features or use  */
/*    of this software must display the following           */
/*    acknowledgement:                                      */
/*      This product includes software developed by the     */
/*      Computer Application Development Group at LAPP      */
/*      Annecy  (Laboratoire d'Annecy le vieux de Physique  */
/*      des Particules - UMR-IN2P3-CNRS).                   */
/* 4. Neither the name of the Institute nor of the          */
/*    Laboratory may be used to endorse or promote products */
/*    derived from this software without specific prior     */
/*    written permission.                                   */
/*                                                          */
/* THIS SOFTWARE IS PROVIDED BY THE LAPP AND CONTRIBUTORS   */
/* ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,         */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE */
/* DISCLAIMED. IN NO EVENT SHALL THE LAPP OR CONTRIBUTORS   */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER   */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY */
/* OF SUCH DAMAGE.                                          */
/************************************************************/

#include <sys/types.h>
#include <stdarg.h>
#include <pthread.h>
#include <CbfHeader.h>

#ifndef CbfHash_h
#define CbfHash_h

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CBFHROOT -1
typedef int (*CbfHPrinter_t) (const char* format, va_list args);

typedef struct CbfHEntry {
  char *key;
  char *data;
  struct CbfHEntry *next; 
} CbfHEntry_t;

typedef struct CbfH {
  CbfHEntry_t **entry;
  int nElem;
  pthread_mutex_t mutex;
} CbfH_t;

extern CbfHEntry_t *CbfHEnter(  CbfH_t *hTable, char *key, void *data );
#define CbfHEnterRoot( k, d ) CbfHEnter( (CbfH_t*)NULL, k, d)
extern void CbfHEntryRemove( CbfHEntry_t *entry );
extern CbfHEntry_t *CbfHFind( CbfH_t *hTable, char *key );
#define CbfHFindRoot( k ) CbfHFind( (CbfH_t*)NULL, k)
extern CbfH_t *CbfHFree( CbfH_t *hTable );
extern CbfHPrinter_t CbfHInstallPrinter( CbfHPrinter_t printer );
extern CbfH_t *CbfHNew( int nElem );
extern void _CbfHPrint(  CbfH_t *hTable, char *tag );
extern void CbfHPrint( CbfH_t *hTable );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* CbfHash_h */
